/*****************************************************************************
* | File      	:   DEV_Config.c
* | Author      :   Waveshare team
* | Function    :   Hardware underlying interface
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-10-10
* | Info        :   Basic version
*
******************************************************************************/
#include "DEV_Config.h"
#include <unistd.h>

uint32_t fd_1;
uint32_t fd_2;

#if USE_DEV_LIB
pthread_t t1;
UWORD pwm_dule=100;
struct sched_param param;
pthread_attr_t attr;
void *BL_PWM(void *arg){
	
	UWORD i=0;
	for(i=0;;i++){
		if(i>64)i=0;
		if(i<(pwm_dule/16))LCD_BL_1;
		else LCD_BL_0;
	}	
	
}
#endif

void DEV_SetBacklight(UWORD Value)
{
	
#ifdef USE_BCM2835_LIB
    bcm2835_pwm_set_data(0,Value);
    
#elif USE_WIRINGPI_LIB
    pwmWrite(LCD_BL,Value);
    
#elif USE_DEV_LIB
	pwm_dule=Value;  
#endif
	
}

/*****************************************
                GPIO
*****************************************/
void DEV_Digital_Write(UWORD Pin, UBYTE Value) //Control GPIO output
{
#ifdef USE_BCM2835_LIB
    bcm2835_gpio_write(Pin, Value);
    
#elif USE_WIRINGPI_LIB
    digitalWrite(Pin, Value);
    
#elif USE_DEV_LIB
    SYSFS_GPIO_Write(Pin, Value);
    
#endif
}

UBYTE DEV_Digital_Read(UWORD Pin) //Get the current status of GPIO
{
    UBYTE Read_value = 0;
#ifdef USE_BCM2835_LIB
    Read_value = bcm2835_gpio_lev(Pin);
    
#elif USE_WIRINGPI_LIB
    Read_value = digitalRead(Pin);
    
#elif USE_DEV_LIB
    Read_value = SYSFS_GPIO_Read(Pin);
#endif
    return Read_value;
}

void DEV_GPIO_Mode(UWORD Pin, UWORD Mode) //Set GPIO working mode
{
#ifdef USE_BCM2835_LIB  
    if(Mode == 0 || Mode == BCM2835_GPIO_FSEL_INPT){
        bcm2835_gpio_fsel(Pin, BCM2835_GPIO_FSEL_INPT);
    }else {
        bcm2835_gpio_fsel(Pin, BCM2835_GPIO_FSEL_OUTP);
    }
#elif USE_WIRINGPI_LIB
    if(Mode == 0 || Mode == INPUT){
        pinMode(Pin, INPUT);
        pullUpDnControl(Pin, PUD_UP);
    }else{ 
        pinMode(Pin, OUTPUT);
    }
#elif USE_DEV_LIB
    SYSFS_GPIO_Export(Pin);
    if(Mode == 0 || Mode == SYSFS_GPIO_IN){
        SYSFS_GPIO_Direction(Pin, SYSFS_GPIO_IN);
    }else{
        SYSFS_GPIO_Direction(Pin, SYSFS_GPIO_OUT);
    }
#endif   
}

/**
 * delay x ms
**/
void DEV_Delay_ms(UDOUBLE xms) // ms
{
#ifdef USE_BCM2835_LIB
    bcm2835_delay(xms);
#elif USE_WIRINGPI_LIB
    delay(xms);
#elif USE_DEV_LIB
    UDOUBLE i;
    for(i=0; i < xms; i++){
        usleep(1000);
    }
#endif
}

static void DEV_GPIO_Init(void)
{
    DEV_GPIO_Mode(LCD_CS, 1);
    DEV_GPIO_Mode(LCD_RST, 1);
    DEV_GPIO_Mode(LCD_DC, 1);
    DEV_GPIO_Mode(LCD_BL, 1);

    DEV_GPIO_Mode(KEY_1, 0);
    DEV_GPIO_Mode(KEY_2, 0);
    DEV_GPIO_Mode(KEY_3, 0);
    DEV_GPIO_Mode(KEY_4, 0);

    LCD_CS_1;
	LCD_BL_1;
}

/******************************************************************************
function:	Module Initialize, the library and initialize the pins, SPI protocol
parameter:
Info:
******************************************************************************/
UBYTE DEV_ModuleInit(void)
{
	
 #ifdef USE_BCM2835_LIB
    if(!bcm2835_init()) {
        printf("bcm2835 init failed  !!! \r\n");
        return 1;
    } else {
        printf("bcm2835 init success !!! \r\n");
    }
	DEV_GPIO_Init();

    printf("USE_SPI\r\n");  
    bcm2835_spi_begin();                                         //Start spi interface, set spi pin for the reuse function
    bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);     //High first transmission
    bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);                  //spi mode 3
    bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_32);  //Frequency
    bcm2835_spi_chipSelect(BCM2835_SPI_CS0);                     //set CE0
    bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);     //enable cs0

    printf("USE_IIC\r\n");
    bcm2835_i2c_begin();	
    bcm2835_i2c_setSlaveAddress(0x3c);
         /**********************************************************/

    
#elif USE_WIRINGPI_LIB  
    if(wiringPiSetupGpio() < 0) { //use BCM2835 Pin number table
        printf("set wiringPi lib failed	!!! \r\n");
        return 1;
    } else {
        printf("set wiringPi lib success  !!! \r\n");
    }
    DEV_GPIO_Init();
    printf("USE_SPI\r\n");  
    wiringPiSPISetup(0,40000000); //Initialize SPI0 channel 0
	pinMode (LCD_BL, PWM_OUTPUT); //Initialize PWM so that the backlight brightness can be adjusted. The pins used for PWM are BCM18 and BCM19.
    pwmWrite(LCD_BL,512); //PWM duty cycle is set to half and full brightness is 1024

    printf("USE_IIC\r\n");
    fd_1 = wiringPiI2CSetup(0x3d); //Initialize 0.96 OLED screen
    fd_2 = wiringPiI2CSetup(0x3c);

   
#elif USE_DEV_LIB
	DEV_GPIO_Init();

    printf("USE_SPI\r\n"); 
    DEV_HARDWARE_SPI_begin("/dev/spidev0.0");

    printf("USE_IIC\r\n");		
    DEV_HARDWARE_I2C_begin("/dev/i2c-1");
    DEV_HARDWARE_I2C_setSlaveAddress(0x3c);

#endif
    return 0;
}

void DEV_SPI_WriteByte(uint8_t Value) 
{
#ifdef USE_BCM2835_LIB
    bcm2835_spi_transfer(Value);
    
#elif USE_WIRINGPI_LIB
    wiringPiSPIDataRW(0,&Value,1); // spi read and write function reads and writes one byte
    
#elif USE_DEV_LIB
    DEV_HARDWARE_SPI_TransferByte(Value);
    
#endif
}

void DEV_SPI_Write_nByte(uint8_t *pData, uint32_t Len)
{
#ifdef USE_BCM2835_LIB
    uint8_t rData[Len];
    bcm2835_spi_transfernb((char *)pData,(char *)rData,Len);
    
#elif USE_WIRINGPI_LIB
    uint8_t Data[Len];
    for (int i = 0; i < Len; i++) {
        Data[i] = pData[i];
    }
    wiringPiSPIDataRW(0, (unsigned char *)Data, Len);// spi read and write function reads and writes len bytes
    
#elif USE_DEV_LIB
    DEV_HARDWARE_SPI_Transfer(pData, Len);
    
#endif
}

void I2C_Write_Byte(uint8_t value, uint8_t Cmd)
{
#ifdef USE_BCM2835_LIB
    char wbuf[2]={Cmd, value};
    bcm2835_i2c_write(wbuf, 2);
#elif USE_WIRINGPI_LIB
	int ref;
    //Pass data to the first screen
    ref = wiringPiI2CWriteReg8(fd_1, (int)Cmd, (int)value); //i2c sends a byte to send information to the fd_1 device
    while(ref != 0) {
        ref = wiringPiI2CWriteReg8 (fd_1, (int)Cmd, (int)value);
        if(ref == 0)
            break;
    }

    //Pass data to the second screen
    ref = wiringPiI2CWriteReg8(fd_2, (int)Cmd, (int)value); //Send information to fd_2 device
    while(ref != 0) {
        ref = wiringPiI2CWriteReg8 (fd_2, (int)Cmd, (int)value);
        if(ref == 0)
            break;
    }
#elif USE_DEV_LIB
    char wbuf[2]={Cmd, value};
    DEV_HARDWARE_I2C_write(wbuf, 2);

#endif
}

/******************************************************************************
function:	Module exits, closes SPI and BCM2835 library
parameter:
Info:
******************************************************************************/
void DEV_ModuleExit(void)
{
#ifdef USE_BCM2835_LIB
    bcm2835_spi_end();
	bcm2835_i2c_end();
    bcm2835_close();


#elif USE_WIRINGPI_LIB
    LCD_CS_0;
	LCD_RST_1;
	LCD_DC_0;

#elif USE_DEV_LIB
    LCD_CS_0;
	LCD_RST_1;
	LCD_DC_0;
    DEV_HARDWARE_SPI_end();
    DEV_HARDWARE_I2C_end();
#endif
}

