import os
import time
import threading
from PIL import Image, ImageSequence
from PIL import ImageFont, ImageDraw
from luma.core.interface.serial import i2c
from luma.core.render import canvas
from luma.oled.device import ssd1306
import RPi.GPIO as GPIO
from time import sleep

# Set GPIO mode to BCM encoding
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)

# Define the BCM pin number of the button
KEY1_PIN = 4
KEY2_PIN = 17
KEY3_PIN = 23
KEY4_PIN = 24

# Set the button pin to input mode and use a pull-up resistor
GPIO.setup(KEY1_PIN, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(KEY2_PIN, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(KEY3_PIN, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(KEY4_PIN, GPIO.IN, pull_up_down=GPIO.PUD_UP)

#Initialize button state
prev_state_key1 = GPIO.input(KEY1_PIN)
prev_state_key2 = GPIO.input(KEY2_PIN)
prev_state_key3 = GPIO.input(KEY3_PIN)
prev_state_key4 = GPIO.input(KEY4_PIN)

#Initialize the first screen
serial_64 = i2c(port=1, address=0x3D) #Initialize the communication interface I2C1, refer to the manual for the I2C address.
device_64 = ssd1306(serial_64, width=128, height=64) #Initialization screen

#Initialize the second screen
serial_32 = i2c(port=1, address=0x3C)
device_32 = ssd1306(serial_32, width=128, height=64)

#Initialize variables that control the screen through keystrokes
flat_1 = 0
flat_2 = 0


def image_gif(): # Open the GIF file and create an Image object for each frame function
    # Open a GIF file and create an Image object for each frame
    frames = []  #Create a buffer to store GIF data and create a frames list
    global flat_1 # Import global variables

    #Convert GIF graphics into screen-readable data
    with Image.open("./cat.gif") as img:
        for frame in ImageSequence.Iterator(img):
            frames.append(frame.convert("1").resize((128,64)))

    while True:
        if flat_1 ==  1:    #Button 1 is pressed
            for frame in frames: #Traverse the data in the frames list to frame
                device_64.display(frame) #Show GIF
                time.sleep(0.01) #Increase CPU usage
                if flat_1== 0 : #Detect button 2 pressed and exit the loop
                    break   
        else:
            device_64.clear() #screen goes off


def image_gif1(): 
    frames = []
    global flat_2
    with Image.open("./cat.gif") as img:
        for frame in ImageSequence.Iterator(img):
            frames.append(frame.convert("1").resize((128,64)))

    while True:
        try:
            if flat_2 ==  1:
                for frame in frames:
                    device_32.display(frame)
                    time.sleep(0.01)
                    if flat_2== 0 :
                        break
            else:
                device_32.clear() 
        except TypeError: #Error handling when TypeError exception is caught
            print() 
            exit()  # Terminate the execution of the program
if __name__ == '__main__': #Used to check if the current script is running as the main program (main module) or as a module that is imported into other scripts
    try:
        # Create thread
        t1 = threading.Thread(target=image_gif1) #t1 thread will execute the image_gif1 function
        t2 = threading.Thread(target=image_gif)  #t2 thread will execute the image_gif function
        t1.daemon=True # Setting the thread as a daemon thread can make it easier for the program to exit when the main thread ends without having to wait for all threads to complete.
        t2.daemon=True
        t1.start() #Start thread
        t2.start()
        #The properties of the daemon thread are only valid before the thread is started.

        while True:
            time.sleep(0.1) #Increase CPU usage
            curr_state_key1 = GPIO.input(KEY1_PIN) #Get key value
            curr_state_key2 = GPIO.input(KEY2_PIN)
            curr_state_key3 = GPIO.input(KEY3_PIN)
            curr_state_key4 = GPIO.input(KEY4_PIN)
            if curr_state_key1 == GPIO.LOW: #Judgment status
                print("KEY1 is Pressed!!!")
                flat_1 = 1 #Screen 1 shows GIF
                while(GPIO.input(KEY1_PIN) == GPIO.LOW): #Hands off detection
                    time.sleep(0.01)
            if curr_state_key2 == GPIO.LOW: 
                print("KEY2 is Pressed!!!")
                flat_1 = 0
                while(GPIO.input(KEY2_PIN) == GPIO.LOW):
                    time.sleep(0.01)
            if curr_state_key3 == GPIO.LOW:
                print("KEY3 is Pressed!!!")
                flat_2 = 1
                while(GPIO.input(KEY3_PIN) == GPIO.LOW):
                    time.sleep(0.01)
            if curr_state_key4 == GPIO.LOW:
                print("KEY4 is Pressed!!!")
                flat_2 = 0
                while(GPIO.input(KEY4_PIN) == GPIO.LOW):
                    time.sleep(0.01)

    except KeyboardInterrupt: #Error handling when KeyboardInterrupt exception is caught
            exit()
        