''' https://luma-lcd.readthedocs.io/en/latest/index.html '''

import time
import logging
from PIL import Image, ImageFont, ImageDraw,ImageSequence
from luma.core.interface.serial import spi
from luma.core.render import canvas
from luma.lcd.device import st7789

logging.basicConfig(level=logging.DEBUG)

serial = spi(port = 0,device = 0,gpio_DC = 22,gpio_RST = 27)
device = st7789(serial, rotate = 0, gpio_LIGHT = 18, active_low=False,width=320, height=240)
device.backlight(True)  # Turn on the backlight

Font1 = ImageFont.truetype("./Font/Font01.ttf",25)
Font2 = ImageFont.truetype("./Font/Font01.ttf",35)
Font3 = ImageFont.truetype("./Font/Font02.ttf",32)

# Create blank image for drawing.
with canvas(device) as draw:
    draw.rectangle((0, 0, 320, 240),  fill="WHITE")
    logging.info("draw point")

    draw.rectangle((5,10,6,11), fill = "BLACK")
    draw.rectangle((5,25,7,27), fill = "BLACK")
    draw.rectangle((5,40,8,43), fill = "BLACK")
    draw.rectangle((5,55,9,59), fill = "BLACK")
    
    logging.info("draw rectangle")
    draw.rectangle([(20,10),(70,60)],fill = "WHITE",outline="BLUE")
    draw.rectangle([(85,10),(130,60)],fill = "BLUE")

    logging.info("draw line")
    draw.line([(20, 10),(70, 60)], fill = "RED",width = 1)
    draw.line([(70, 10),(20, 60)], fill = "RED",width = 1)
    draw.line([(170,15),(170,55)], fill = "RED",width = 1)
    draw.line([(150,35),(190,35)], fill = "RED",width = 1)

    logging.info("draw circle")
    draw.arc((150,15,190,55),0, 360, fill =(0,255,0))
    draw.ellipse((150,65,190,105), fill = (0,255,0))

    logging.info("draw text")

    draw.rectangle([(0,65),(140,100)],fill = "WHITE")
    draw.text((5, 68), 'Hello world', fill = "BLACK",font=Font1)
    draw.rectangle([(0,115),(190,160)],fill = "RED")
    draw.text((5, 118), 'WaveShare', fill = "WHITE",font=Font2)
    draw.text((5, 160), '1234567890', fill = "GREEN",font=Font3)
    text= u"微雪电子"
    draw.text((5, 200),text, fill = "BLUE",font=Font3)

time.sleep(3)

logging.info("show image")
image = Image.open('./LCD_2inch.bmp')	
device.display(image)

logging.info("Press any key to exit")
input()

